<?php
/**
 * 2018-2023 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2023 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
 
/**
* ###### DUPLICATE & CUSTOMIZE 1/3: 
*        you are customizing "mxb_cookiebox_consent"
*        where "consent" is your [customname]
*        if you want to create "mxb_cookiebox_bestseller", "bestseller" will be your [customname]
*        [customname] in file and folder name MUST be a string without underscores and in lowercase. YES: "bestseller" - NO: "BestSeller", "best_seller", "best-seller", "bestseller22"
*        [Customname] in Class name MUST be capitalized. YES: "Customname", "Bestseller" ....
*        [special extensions tpl_, js_, css_, translations/*.php_]: that files are ready to be installed with the customized name of the module. You can change it as you wish but remember to leave the "_mxb_[module]_" key untouched: it will be substituted with the right module name during the installation.
*
*        remember to change:
*            the module folder name to "mxb_cookiebox_[customname]"
*              file "mxb_cookiebox_[customname].php":
*                the class name "class Mxb_Cookiebox_[Customname]"
*                the CUSTOM_NAME const value in class declaration
*            file "config.xml":
*                the module name value to "mxb_cookiebox_[customname]"
*/

/**
 * for DEBUG: if you use mxb_tools module, you'll find the log file in folder: /module/mxb_tools/log/
 *         require_once _PS_MODULE_DIR_.'mxb_tools/classes/Mxb_ToolsDebug.php';
 * and then use this function:
 *        Mxb_ToolsDebug::WriteLog($classname, $function, $message, $bypass_settings = false)
 *    i.e: 
 *        Mxb_ToolsDebug::WriteLog($this->name, 'className::__construct()', 'START', true);
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

 /**
  * ###### DUPLICATE & CUSTOMIZE 2/3: class declaration: change the class name as the module name: Mxb_Cookiebox_[Customname] 
  */
class Mxb_Cookiebox_Consent extends Module // implements WidgetInterface
{
    /**
    * ###### DUPLICATE 3/3: set CUSTOM_NAME as "yourname" in lowercase
    *                       set $mainHook as you wish: it is the default hook for this module
    */
    const CUSTOM_NAME   = 'consent';
    public $mainHook    = 'displayTop';    
    
    /**
       * ###### DUPLICATE & CUSTOMIZE: no more changes are needed from here
     */

    const MODULE_NAME_ROOT = 'mxb_cookiebox';
    public $documentationUrl = 'https://www.mxbinformatica.it/prestashop-mxb-cookiebox/';
    public $thumbnailUrl = 'https://www.mxbinformatica.it/ps_modules/thumbs/mxb_cookiebox.jpg';

    private $templateFile;

    public function __construct()
    {
        // not used ...
        $this->disableMobileDevice  = false;
        $this->registerModuleJs     = true;
        
        $this->customName       = self::CUSTOM_NAME;
        $this->rootName         = self::MODULE_NAME_ROOT . '_';
        $this->name             = $this->rootName . $this->customName;
        $this->displayName      = 'MXB Cookie: ' . $this->customName;
        $this->templateFile     = 'front/mxb_cookiebox.tpl';
        $this->version          = '1.0.1';
        $this->author           = 'MXB Informatica';
        $this->need_instance    = 0;
        $this->bootstrap         = true;
        parent::__construct();

        $this->description              = $this->l('Displays a cookie consent banner on your shop. You can always move this module to another hook if you like.');
        $this->ps_versions_compliancy   = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);
        $this->customCssFolder          = "customcss";
        $this->cookie_desc              = $this->name . '_DESC';
        $this->cookie_btn_ok            = $this->name . '_BTN_OK';
        $this->cookie_btn_cancel        = $this->name . '_BTN_CANCEL';
        $this->cookie_link_privacy      = $this->name . '_LINK_PRIVACY';
        $this->cookie_link_privacy_desc = $this->name . '_LINK_PRIVACY_DESC';
        $this->cookie_css_enabled       = $this->name . '_CSS_ENABLED';
        $this->cookie_css_content       = $this->name . '_CSS_CONTENT';
        // $this->installModuleFiles(); // TODO: delete this in production version (it's already called in install() method !!!)
    }

    public function install()
    {
        $res = (parent::install() &&
            $this->registerHook($this->mainHook) &&
            /* some usefull hooks ...
            $this->registerHook('displayHome') &&
            $this->registerHook('displayTop') &&

            $this->registerHook('displayLeftColumn') &&
            $this->registerHook('displayRightColumn') &&
            $this->registerHook('displayTopColumn') &&
            $this->registerHook('displayWrapperBottom') &&
            $this->registerHook('displayWrapperTop') &&
            $this->registerHook('displayMaintenance') &&

            $this->registerHook('displayInvoice') &&
            $this->registerHook('displayorderConfirmation') &&
            $this->registerHook('displayorderConfirmation2') &&
            $this->registerHook('displayShoppingCart') &&
            $this->registerHook('displayShoppingCartFooter') &&
            $this->registerHook('displayOrderDetail') &&
            $this->registerHook('displayPaymentTop') &&

            $this->registerHook('displayNav') &&
            $this->registerHook('displayNav1') &&
            $this->registerHook('displayNav2') &&
            $this->registerHook('displayNavFullWidth') &&

            $this->registerHook('displayFooter') &&
            $this->registerHook('displayFooterBefore') &&

            $this->registerHook('displayFooterProduct') &&
            $this->registerHook('displayAfterProductThumbs') &&
            $this->registerHook('displayLeftColumnProduct') &&
            $this->registerHook('displayRightColumnProduct') &&
            $this->registerHook('displayProductAdditionalInfo') &&
            $this->registerHook('displayProductExtraContent') &&
            $this->registerHook('displayProductPrioceBlock') &&
            */
            $this->registerHook('actionObjectLanguageAddAfter') &&
            $this->installFixtures());

            // if ($this->registerModuleJs)
            $res &= $this->registerHook('actionFrontControllerSetMedia');
            $res &= $this->registerHook('header');
            // if ($this->disableMobileDevice)
            // $res &= $this->disableDevice(Context::DEVICE_MOBILE);

            return $res;
    }

    public function hookActionObjectLanguageAddAfter($params)
    {
        return $this->installFixture(
            (int)$params['object']->id, 
            Configuration::get($this->cookie_desc, (int)Configuration::get('PS_LANG_DEFAULT')),
            Configuration::get($this->cookie_link_privacy_desc, (int)Configuration::get('PS_LANG_DEFAULT'))
        );
    }

    protected function installFixtures()
    {
        $languages = Language::getLanguages(false);
        foreach ($languages as $lang) {
            $this->installFixture((int)$lang['id_lang'], 'Please accept our privacy policy', 'click to read our privacy policy');
        }
        $this->installModuleFiles();
        return true;
    }

    protected function installFixture($id_lang, $desc = '', $link_desc = '')
    {
        $values[$this->cookie_desc][(int)$id_lang]                 = $desc;
        $values[$this->cookie_btn_ok][(int)$id_lang]             = 'Ok';
        $values[$this->cookie_btn_cancel][(int)$id_lang]         = 'Cancel';
        $values[$this->cookie_link_privacy][(int)$id_lang]         = $this->context->link->getPageLink('index');
        $values[$this->cookie_link_privacy_desc][(int)$id_lang] = $link_desc;

        Configuration::updateValue($this->cookie_desc,                     $values[$this->cookie_desc]);
        Configuration::updateValue($this->cookie_btn_ok,                 $values[$this->cookie_btn_ok]);
        Configuration::updateValue($this->cookie_btn_cancel,             $values[$this->cookie_btn_cancel]);
        Configuration::updateValue($this->cookie_link_privacy,             $values[$this->cookie_link_privacy]);
        Configuration::updateValue($this->cookie_link_privacy_desc,     $values[$this->cookie_link_privacy_desc]);

        if (Configuration::get($this->cookie_css_enabled) === false) {
            Configuration::updateValue($this->cookie_css_enabled,         false);
        }
    }

    public function uninstall()
    {
        Configuration::deleteByName($this->cookie_desc);
        Configuration::deleteByName($this->cookie_btn_ok);
        Configuration::deleteByName($this->cookie_btn_cancel);
        Configuration::deleteByName($this->cookie_link_privacy);
        Configuration::deleteByName($this->cookie_link_privacy_desc);
        Configuration::deleteByName($this->cookie_css_enabled);
        // Configuration::deleteByName($this->cookie_css_content);

        return parent::uninstall();
    }

    public function postProcess()
    {
        if (Tools::isSubmit('submitStoreConf')) {
            $languages = Language::getLanguages(false);
            $values = array();
            //$update_images_values = false;

            foreach ($languages as $lang) {
                $values[$this->cookie_desc][$lang['id_lang']]                 = Tools::getValue($this->cookie_desc . '_'.$lang['id_lang']);
                $values[$this->cookie_btn_ok][$lang['id_lang']]             = Tools::getValue($this->cookie_btn_ok . '_'.$lang['id_lang']);
                $values[$this->cookie_btn_cancel][$lang['id_lang']]         = Tools::getValue($this->cookie_btn_cancel . '_'.$lang['id_lang']);
                $values[$this->cookie_link_privacy][$lang['id_lang']]         = Tools::getValue($this->cookie_link_privacy . '_'.$lang['id_lang']);
                $values[$this->cookie_link_privacy_desc][$lang['id_lang']]     = Tools::getValue($this->cookie_link_privacy_desc . '_'.$lang['id_lang']);
            }

            // for html string: Configuration::updateValue($key, $values, $html = true, $id_shop_group = null, $id_shop = null)
            Configuration::updateValue($this->cookie_desc,                 $values[$this->cookie_desc]);
            Configuration::updateValue($this->cookie_btn_ok,             $values[$this->cookie_btn_ok]);
            Configuration::updateValue($this->cookie_btn_cancel,         $values[$this->cookie_btn_cancel]);
            Configuration::updateValue($this->cookie_link_privacy,         $values[$this->cookie_link_privacy]);
            Configuration::updateValue($this->cookie_link_privacy_desc, $values[$this->cookie_link_privacy_desc]);
            Configuration::updateValue($this->cookie_css_enabled,         Tools::getValue($this->cookie_css_enabled));

            // save css content
            $this->saveCssFile(Tools::getValue($this->cookie_css_content));
            $this->_clearCache($this->templateFile);

            return $this->displayConfirmation($this->trans('The settings have been updated.', array(), 'Admin.Notifications.Success'));
        }

        return '';
    }

    public function getContent()
    {
        return $this->postProcess().$this->renderForm();
    }

    public function renderForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Settings', array(), 'Admin.Global'),
                    'icon' => 'icon-cogs'
                ),
                'description' => $this->description
                                . '<br><b>' . $this->l('Documentation') . '</b>: <a href="https://www.mxbinformatica.it/prestashop-mxb-cookiebox/" target="_blank">' . $this->l('click here') . '</a>',
                
                'input' => array(
                    array(
                        'type' => 'text',
                        'lang' => true,
                        'label' => $this->l('Privacy page link'),
                        'name' => $this->cookie_link_privacy,
                        'desc' => $this->l('Enter the link for the privacy document available for users. When clicking on the link, the page opens in a new window. If no link is entered, it redirects to the homepage.')
                    ),
                    array(
                        'type' => 'text',
                        'lang' => true,
                        'label' => $this->l('Privacy link text'),
                        'name' => $this->cookie_link_privacy_desc,
                        'desc' => $this->l('Please enter a short text the privacy link.')
                    ),
                    array(
                        'type' => 'textarea',
                        'lang' => true,
                        'label' => $this->l('Cookie consent text'),
                        'name' => $this->cookie_desc,
                        'desc' => $this->l('Please enter a short text for cookie consent.')
                    ),
                    array(
                        'type' => 'text',
                        'lang' => true,
                        'label' => $this->l('Accept button text'),
                        'name' => $this->cookie_btn_ok,
                        'desc' => $this->l('Please enter a short text for the accept button.')
                    ),
                    array(
                        'type' => 'text',
                        'lang' => true,
                        'label' => $this->l('Close button text'),
                        'name' => $this->cookie_btn_cancel,
                        'desc' => $this->l('Please enter a short text for the close button. If no text is entered, the X button in the top-right corner will be used.')
                    ),
                        
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Enable custom CSS'),
                         'name' => $this->cookie_css_enabled,
                         'required' => false,
                         'class' => 't',
                         'is_bool' => true,
                         'values' => array(
                                array(
                                    'id' => 'active_on',
                                    'value' => 1,
                                    'label' => $this->trans('Enabled', array(), 'Admin.Global'),
                                     ),
                                array(
                                    'id' => 'active_off',
                                    'value' => 0,
                                    'label' => $this->trans('Disabled', array(), 'Admin.Global'),
                                     ),
                           ),
                        'hint' => $this->l('When this option is selected, the following custom CSS will be used instead the default one.'),
                    ),
                    array(
                        'type' => 'textarea',
                        'lang' => false,
                        'label' => $this->l('Custom CSS'),
                        'name' => $this->cookie_css_content,
                        'desc' => $this->l('You can modify this precompiled css to change colors, dimensions and other css parameters of the banner, only for this module.')
                                            . '<br><br><b>' . $this->l('Download DEFAULT CSS file') 
                                            . ': <a href="' . $this->getModulePathUri() . 'views/css/custom.css" target="_new">custom.css</a></b>'
                    ),
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Actions')
                )
            ),
        );

        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->default_form_language = $lang->id;
        $helper->module = $this;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitStoreConf';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'uri' => $this->getPathUri(),
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValues()
    {
        $languages = Language::getLanguages(false);
        $fields = array();

        //$fields[$this->banner_type] = Tools::getValue($this->banner_type, Configuration::get($this->banner_type));
        foreach ($languages as $lang) {
            $fields[$this->cookie_desc][$lang['id_lang']]                 = Tools::getValue($this->cookie_desc . '_'.$lang['id_lang'],                 Configuration::get($this->cookie_desc, $lang['id_lang']));
            $fields[$this->cookie_btn_ok][$lang['id_lang']]             = Tools::getValue($this->cookie_btn_ok . '_'.$lang['id_lang'],                 Configuration::get($this->cookie_btn_ok, $lang['id_lang']));
            $fields[$this->cookie_btn_cancel][$lang['id_lang']]         = Tools::getValue($this->cookie_btn_cancel . '_'.$lang['id_lang'],             Configuration::get($this->cookie_btn_cancel, $lang['id_lang']));
            $fields[$this->cookie_link_privacy_desc][$lang['id_lang']]     = Tools::getValue($this->cookie_link_privacy_desc . '_'.$lang['id_lang'],     Configuration::get($this->cookie_link_privacy_desc, $lang['id_lang']));
            
            // check link
            $cookie_link_privacy = Tools::getValue($this->cookie_link_privacy . '_'.$lang['id_lang'],         Configuration::get($this->cookie_link_privacy, $lang['id_lang']));
            if (empty($cookie_link_privacy)) {
                $cookie_link_privacy = $this->context->link->getPageLink('index');
            }
            $fields[$this->cookie_link_privacy][$lang['id_lang']]         = $cookie_link_privacy;
            
        }
        $fields[$this->cookie_css_enabled]     = Tools::getValue($this->cookie_css_enabled, Configuration::get($this->cookie_css_enabled));
        
        // get custom css
        $fields[$this->cookie_css_content]     = $this->readCssFile();
        
        return $fields;
    }

    public function getConfigValues($hookName = '')
    {
        $formValues = $this->getConfigFieldsValues();
        $idLang = $this->context->language->id;
        
        return array(
            'cookie_link_privacy'       => $this->updateUrl($formValues[$this->cookie_link_privacy][$idLang]),
            'cookie_link_privacy_desc'  => $formValues[$this->cookie_link_privacy_desc][$idLang],
            'cookie_btn_ok'             => $formValues[$this->cookie_btn_ok][$idLang],
            'cookie_btn_cancel'         => $formValues[$this->cookie_btn_cancel][$idLang],
            'cookie_desc'               => $formValues[$this->cookie_desc][$idLang],
            'cookie_css_content'        => $formValues[$this->cookie_css_content],
            'hookName'                  => $hookName,
        );
    }

    private function updateUrl($link)
    {
        if (substr($link, 0, 7) !== "http://" && substr($link, 0, 8) !== "https://") {
            $link = "http://" . $link;
        }

        return $link;
    }

     /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader() 
    {
        $this->context->controller->addJquery();
        $this->context->controller->addJS($this->_path.'/views/js/front.js');
        //$this->context->controller->addJS($this->_path.'/views/js/module.js');
        $this->context->controller->addJS($this->_path.'/views/js/cookiebox.js');
        $this->context->controller->addCSS($this->_path.'/views/css/front.css');
        //$this->context->controller->addCSS($this->_path.'/views/css/cookiebox.css');
        
        if ((bool) Configuration::get($this->cookie_css_enabled)) {
            $this->context->controller->addCSS($this->_path.'/views/'.$this->customCssFolder.'/custom.css');
        } else {
            $this->context->controller->addCSS($this->_path.'/views/css/cookiebox.css');
        }
    }

    /**
     * set usefull values into smarty to make it available for js
     * hidden input fields must be created in view/templates/hook/gadgetcard-header.tpl to be available to JS (childclassic > custom.js)
     * Located in:
     *    /themes/classic/templates/_partials/header.tpl
     *    /themes/classic/templates/checkout/_partials/header.tpl
     */
    public function hookDisplayTop($hookParams)
    {
        $this->context->smarty->assign('mxbcbValues', $this->getConfigValues($this->mainHook));
             
         return $this->display(__FILE__, $this->templateFile);
        
    }

    public function installModuleFiles()
    {
        $ret = $this->installModuleFile('translations/it.php');
        $ret &= $this->installModuleFile('views/css/custom.css');
        return $ret;
    }

    public function installModuleFile($module_file = 'translations/it.php', $overwrite = true)
    {
        $filedir = _PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR;
        $filename_src = $filedir . $module_file . '_'; // add underscore to open the source unmodifyed file
        $filename_tgt = $filedir . $module_file;
        $searchString = '_' . $this->rootName;
        
        if (!file_exists($filename_src)) {
            return false;
        }
        if (file_exists($filename_tgt) && !$overwrite) {
            return false;
        }

        $myfile = fopen($filename_src, "r");
        $filesize = filesize($filename_src);
        if ((int)$filesize === 0) {
            return false;
        }
        $content = fread($myfile, $filesize);
        fclose($myfile);
        
        // change default class with custom class and save file
        // text to replace not found ?
        // if (strpos($content, $searchString) === false)
        //     return true;
        // already replaced (replace text is already present) ?
        if (strpos($content, $this->name) === false) {
            $content = str_replace($searchString, $this->name, $content);
        }
        // update the real file
        $myfile = fopen($filename_tgt, "w");
        fwrite($myfile, $content);
        fclose($myfile);

        return true;
    }

    protected function saveCssFile($content, $cssfile = 'custom.css')
    {
        $filename = _PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.$this->customCssFolder.DIRECTORY_SEPARATOR . $cssfile;
        $myfile = fopen($filename, "w");
        fwrite($myfile, $content);
        fclose($myfile);
    }
        
    protected function readCssFile($cssfile = 'custom.css')
    {
        // check if file exists and create it if necessary
        $filename = _PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.$this->customCssFolder.DIRECTORY_SEPARATOR . $cssfile;
        if (!file_exists($filename)) {
            $this->installModuleFile('views'.DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR.$cssfile, false);
            copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR . $cssfile, $filename);
        }
        $myfile = fopen($filename, "r");
        $content = fread($myfile, filesize($filename));
        fclose($myfile);

        return $content;
    }
   
    public function getModulePathUri()
    {
        // $uri =  Tools::getHttpHost(true).__PS_BASE_URI__ . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . $this->name . DIRECTORY_SEPARATOR;    // first method: OK
        $uri = $this->path2url($this->getModulePath());            // second method: OK
        if (substr($uri, -1) != DIRECTORY_SEPARATOR) {
            $uri .= DIRECTORY_SEPARATOR;
        }

        return $uri;
    }

    public function getModulePath()
    {
        return _PS_MODULE_DIR_ . $this->name . DIRECTORY_SEPARATOR;
    }

    public function path2url($file, $Protocol=null)
    {
        if ($Protocol === null) {
            $Protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . '://';        
        }
        // Mxb_ToolsDebug::WriteLog('mxb_tools', 'Mxb_ToolsUtils::path2url', 'file: '.$file.', HTTP_HOST:'.$_SERVER['HTTP_HOST'].', DOCUMENT_ROOT:'.$_SERVER['DOCUMENT_ROOT']);
        $DOCUMENT_ROOT = $_SERVER['DOCUMENT_ROOT'];
        if (substr($DOCUMENT_ROOT, -1) != DIRECTORY_SEPARATOR) {
            $DOCUMENT_ROOT .= DIRECTORY_SEPARATOR;
        }
        $HTTP_HOST = $_SERVER['HTTP_HOST'];
        if (substr($HTTP_HOST, -1) != DIRECTORY_SEPARATOR) {
            $HTTP_HOST .= DIRECTORY_SEPARATOR;
        }

        return $Protocol.$HTTP_HOST.str_replace($DOCUMENT_ROOT, '', $file);
    }
}

