/**
 * 2018-2023 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2023 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

 /*
 1) link this cookiebox.js in your html page

 2) also link cookiebox.css

 3) insert a div wherever you want (i.e. at the end of body)
         <div class="cookieBox">aaa</div>

 4) insert the call to showCookieBox(cookieText, dismissText, linkText, linkHref, newWindow) in the document ready function, compiling it with your text
        <script>
            document.addEventListener("DOMContentLoaded", function(event) {
                cookieBox.showCookieBox("cookies policy privacy text", "Accept and close", "Our privacy policy", "policy.html", false);
              });
        </script>
 */
 
(function(window) {

      if (!!window.cookieBox) {
        return window.cookieBox;
      }

    var document = window.document;
      // IE8 does not support textContent, so we should fallback to innerText.
      //var supportsTextContent = 'textContent' in document.body;

      var cookieBox = (function() {
    var cookieName = 'displayCookieConsent';
    
    var cookieText, dismissText, linkText, linkHref, newWindow, callback;

    function setCookieBox(_cookieText, _dismissText, _linkText, _linkHref, _newWindow, _callback) {
        /*
        if (_cookieText)     cookieText    = _cookieText;
        if (_dismissText)     dismissText    = _dismissText;
        if (_linkText)         linkText    = _linkText;
        if (_linkHref)         linkHref    = _linkHref;
        if (_newWindow)        newWindow    = _newWindow;
        if (_callback)        callback    = _callback;
        */
        return createCookieBox(_cookieText, _dismissText, _linkText, _linkHref, _newWindow, _callback);
    }

    function cookieCallback() {
        callback();
    }


    function createCookieBox(cookieText, dismissText, linkText, linkHref, newWindow, callback) {
    //    setCookieBox(_cookieText, _dismissText, _linkText, _linkHref, _newWindow);
        
        if (!_shouldDisplayConsent())
            return null;
        
        var box = document.getElementsByClassName("cookieBox")[0];
        var str = '<div class="cookieBox-text" id="cookieBox-text">' + cookieText + '</div>'
            + '<div class="cookieBox-text">';
        if (callback) {
            str += '<span class="cookieBox-button" id="cookieBox-btn-show" onClick="cookieBox.cookieCallback();">';
        } else {
            str +=  (newWindow ?
                    '<span class="cookieBox-button" id="cookieBox-btn-show" onClick="window.open(\'' + linkHref + '\');">'
                            :
                    '<span class="cookieBox-button" id="cookieBox-btn-show" onClick="document.location=\'' + linkHref + '\';">');
        }
        str += linkText + '</span>'
                + '<span class="cookieBox-button" id="cookieBox-btn-close" onClick="cookieBox.dismissCookieBox();">' + dismissText + '</span>'
            + '</div>';
        box.innerHTML = str;
        
        return box;
    }


    function showCookieBox(_cookieText, _dismissText, _linkText, _linkHref, _newWindow, _callback) {
        
        var box = createCookieBox(_cookieText, _dismissText, _linkText, _linkHref, _newWindow, _callback);
        
        if (box == null)
            return;
        /*
        setCookieBox(_cookieText, _dismissText, _linkText, _linkHref, _newWindow);
        
        //if (!_shouldDisplayConsent())
        //    return;
        
        var box = document.getElementsByClassName("cookieBox")[0];
        var str = '<div class="cookieBox-text" id="cookieBox-text">' + cookieText + '</div>'
            + '<div class="cookieBox-text">';
        if (callback) {
            str += '<span class="cookieBox-button" id="cookieBox-btn-show" onClick="cookieBox.cookieCallback();">';
        } else {
            str +=  (newWindow ?
                    '<span class="cookieBox-button" id="cookieBox-btn-show" onClick="window.open(\'' + linkHref + '\');">'
                            :
                    '<span class="cookieBox-button" id="cookieBox-btn-show" onClick="document.location=\'' + linkHref + '\';">');
        }
        str += linkText + '</span>'
                + '<span class="cookieBox-button" id="cookieBox-btn-close" onClick="cookieBox.dismissCookieBox();">' + dismissText + '</span>'
            + '</div>';
        box.innerHTML = str;
        */
        setTimeout(function () {
              // Wait 2 seconds before show the box
              box.style.display = "inline-block";
            }, 1000);
        }

        function dismissCookieBox() {
              _saveUserPreference();
              document.getElementsByClassName('cookieBox')[0].classList.add("hidden");
              return false;
        }
        

        function _saveUserPreference() {
            // Set the cookie expiry to one year after today.
            var expiryDate = new Date();
            expiryDate.setFullYear(expiryDate.getFullYear() + 1);
            document.cookie = cookieName + '=y; expires=' + expiryDate.toGMTString();
        }

        function _shouldDisplayConsent() {
            // Display the header only if the cookie has not been set.
            return !document.cookie.match(new RegExp(cookieName + '=([^;]+)'));
        }

        var exports = {};
        exports.showCookieBox         = showCookieBox;
        exports.setCookieBox         = setCookieBox;
        exports.dismissCookieBox     = dismissCookieBox;
        exports.cookieCallback         = cookieCallback;
        return exports;
    })();

      window.cookieBox = cookieBox;
      return cookieBox;
})(this);
